@@ -19,7 +19,7 @@
  *
  */
 
-#undef LOCAL_DEBUG
+#define LOCAL_DEBUG
 #ifdef _WIN32
 #include "win32/config.h"
 #else
@@ -152,6 +152,7 @@ void set_xml_font_manager( ASFontManager *fontman )
 	_as_xml_font_manager = fontman ;
 }
 
+
 void asxml_var_insert(const char* name, int value);
 
 void
@@ -160,7 +161,7 @@ asxml_var_init(void)
 	int w, h;
 	if ( asxml_var == NULL )
 	{
-    	asxml_var = create_ashash(0, string_hash_value, string_compare, string_destroy);
+    	asxml_var = create_ashash(0, string_hash_value, string_compare, string_destroy_without_data);
     	if (!asxml_var) return;
     	if (dpy != NULL && GetRootDimensions(&w, &h))
 		{
@@ -183,7 +184,7 @@ asxml_var_insert(const char* name, int value)
 
     show_progress("Defining var [%s] == %d.", name, value);
 
-    hdata.iptr = NEW(int); *(hdata.iptr) = value;
+    hdata.i = value;
     add_hash_item(asxml_var, AS_HASHABLE(mystrdup(name)), hdata.vptr);
 }
 
@@ -194,10 +195,12 @@ asxml_var_get(const char* name)
 
     if (!asxml_var) asxml_var_init();
     if (!asxml_var) return 0;
-    get_hash_item(asxml_var, AS_HASHABLE(name), &hdata.vptr);
-	if (!hdata.iptr)
+    if( get_hash_item(asxml_var, AS_HASHABLE(name), &hdata.vptr) != ASH_Success ) 
+	{	
 		show_debug(__FILE__, "asxml_var_get", __LINE__, "Use of undefined variable [%s].", name);
-    return hdata.iptr ? *hdata.iptr : 0;
+		return 0;
+	}
+    return hdata.i;
 }
 
 int
@@ -266,6 +269,7 @@ compose_asimage_xml(ASVisual *asv, ASImageManager *imman, ASFontManager *fontman
 	xml_elem_t* doc;
 	ASImageManager *my_imman = imman, *old_as_xml_imman = _as_xml_image_manager ;
 	ASFontManager  *my_fontman = fontman, *old_as_xml_fontman = _as_xml_font_manager ; ;
+	int my_imman_curr_dir_path_idx = MAX_SEARCH_PATHS ;
 
     asxml_var_init();
 
@@ -280,13 +284,30 @@ compose_asimage_xml(ASVisual *asv, ASImageManager *imman, ASFontManager *fontman
 	if (doc)
 	{
 		xml_elem_t* ptr;
+		Bool local_dir_included = False ;
 		if( my_imman == NULL )
 		{	
 			if( _as_xml_image_manager == NULL )
+			{
+				local_dir_included	  = True ;
 				_as_xml_image_manager = create_generic_imageman( path );/* we'll want to reuse it in case of recursion */
+			}
 			my_imman = _as_xml_image_manager ;
 		}
 
+		if( !local_dir_included )
+		{
+			register int i = 0;
+			char **paths = my_imman->search_path ;
+			while( i < MAX_SEARCH_PATHS && paths[i] != NULL ) ++i;
+			if( i < MAX_SEARCH_PATHS ) 
+			{	
+				paths[i] = mystrdup(path) ;			
+				paths[i+1] = NULL ;
+				my_imman_curr_dir_path_idx = i ;
+			}
+		}	 
+
 		if( my_fontman == NULL )
 		{
 			if( _as_xml_font_manager == NULL )
@@ -299,7 +320,13 @@ compose_asimage_xml(ASVisual *asv, ASImageManager *imman, ASFontManager *fontman
 			if (tmpim && im) safe_asimage_destroy(im);
 			if (tmpim) im = tmpim;
 		}
-LOCAL_DEBUG_OUT( "result im = %p, im->imman	= %p, my_imman = %p, im->magic = %8.8X", im, im?im->imageman:NULL, my_imman, im?im->magic:0 );
+LOCAL_DEBUG_OUT( "result im = %p, im->imman	= %p, my_imman = %p, im->magic = %8.8lX", im, im?im->imageman:NULL, my_imman, im?im->magic:0 );
+		
+		if( my_imman_curr_dir_path_idx < MAX_SEARCH_PATHS ) 
+		{
+			free(my_imman->search_path[my_imman_curr_dir_path_idx]);
+			my_imman->search_path[my_imman_curr_dir_path_idx] = NULL ;			
+		}
 
 		if( my_imman != imman && my_imman != old_as_xml_imman )
 		{/* detach created image from imman to be destroyed : */
@@ -307,6 +334,7 @@ LOCAL_DEBUG_OUT( "result im = %p, im->imman	= %p, my_imman = %p, im->magic = %8.
 				forget_asimage( im );
 			destroy_image_manager(my_imman, False);
 		}
+
 		if( my_fontman != fontman && my_fontman != old_as_xml_fontman  )
 			destroy_font_manager(my_fontman, False);
 		/* must restore managers to its original state */
@@ -316,7 +344,7 @@ LOCAL_DEBUG_OUT( "result im = %p, im->imman	= %p, my_imman = %p, im->magic = %8.
 		/* Delete the xml. */
 		xml_elem_delete(NULL, doc);
 	}
-	LOCAL_DEBUG_OUT( "returning im = %p, im->imman	= %p, im->magic = %8.8X", im, im?im->imageman:NULL, im?im->magic:0 );
+	LOCAL_DEBUG_OUT( "returning im = %p, im->imman	= %p, im->magic = %8.8lX", im, im?im->imageman:NULL, im?im->magic:0 );
 	return im;
 }
 
@@ -456,6 +484,53 @@ double parse_math(const char* str, char** endptr, double size) {
 	return total;
 }
 
+typedef struct ASImageXMLState
+{
+	ASFlagType 		flags ;
+ 	ASVisual 		*asv;
+	ASImageManager 	*imman ;
+	ASFontManager 	*fontman ;
+	
+}ASImageXMLState;
+
+
+ASImage *commit_xml_image_built( ASImageXMLState *state, char *id, ASImage *result )
+{	
+	if (state && id && result) 
+	{
+    	char* buf = NEW_ARRAY(char, strlen(id) + 1 + 6 + 1);
+		show_progress("Storing image id [%s] with image manager %p .", id, state->imman);
+        sprintf(buf, "%s.width", id);
+        asxml_var_insert(buf, result->width);
+        sprintf(buf, "%s.height", id);
+        asxml_var_insert(buf, result->height);
+        free(buf);
+		if( result->imageman != NULL )
+		{
+			ASImage *tmp = clone_asimage(result, SCL_DO_ALL );
+			safe_asimage_destroy(result );
+			result = tmp ;
+		}
+		if( result )
+		{
+			if( !store_asimage( state->imman, result, id ) )
+			{
+				show_warning("Failed to store image id [%s].", id);
+				safe_asimage_destroy(result );
+				result = fetch_asimage( state->imman, id );
+				/*show_warning("Old image with the name fetched as %p.", result);*/
+			}else
+			{
+				/* normally generated image will be destroyed right away, so we need to
+			 	* increase ref count, in order to preserve it for future uses : */
+				dup_asimage( result );
+			}
+		}
+	}
+	return result;
+}
+
+
 
 /* Each tag is only allowed to return ONE image. */
 ASImage*
@@ -465,6 +540,12 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 	char* id = NULL;
 	ASImage* result = NULL;
 
+	ASImageXMLState state ; 
+
+	memset( &state, 0x00, sizeof(state));
+	state.asv = asv ; 
+	state.imman = imman ; 
+	state.fontman = fontman ; 
 /****** libAfterImage/asimagexml/img
  * NAME
  * img - load image from the file.
@@ -747,7 +828,8 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
  *       opacity="value" replace="0|1" delay="mlsecs">
  * ATTRIBUTES
  * id       Optional.  Image will be given this name for future reference.
- * dst      Required.  Name of file image will be saved to.
+ * dst      Optional.  Name of file image will be saved to. If omitted
+ *          image will be dumped into stdout - usefull for CGI apps.
  * format   Optional.  Ouput format of saved image.  Defaults to the
  *          extension of the "dst" parameter.  Valid values are the
  *          standard AS image file formats: xpm, jpg, png, gif, tiff.
@@ -1044,6 +1126,10 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 					}
 				}
 				result = make_gradient(asv, &gradient, width, height, SCL_DO_ALL, ASA_ASImage, 0, ASIMAGE_QUALITY_DEFAULT);
+				if( gradient.color ) 
+					free( gradient.color );
+				if( gradient.offset )
+					free( gradient.offset );
 			}
 		}
 		if (rparm) *rparm = parm; else xml_elem_delete(NULL, parm);
@@ -1822,10 +1908,14 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 #define  ASXML_ALIGN_BOTTOM (0x01<<3)
 			int *align ;
 			int i ;
+			merge_scanlines_func op_func = blend_scanlines_name2func(pop);
 
+			if( op_func == NULL ) 
+				op_func = alphablend_scanlines ;
 			/* Build the layers first. */
 			layers = create_image_layers( num );
 			align = safecalloc( num, sizeof(int));
+
 			for (num = 0, ptr = doc->child ; ptr ; ptr = ptr->next) 
 			{
 				int x = 0, y = 0;
@@ -1902,7 +1992,7 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 					layers[num].clip_height = clip_height ;
 					layers[num].tint = tint;
 					layers[num].bevel = 0;
-					layers[num].merge_scanlines = blend_scanlines_name2func(pop);
+					layers[num].merge_scanlines = op_func;
 					if( clip_width + x > 0 )
 					{
 						if( width < clip_width + x )
@@ -2004,7 +2094,7 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 		
 		if( format != NULL ) 
 		{	
-			char *interpreted_format = interpret_ctrl_codes( strdup(format) );
+			char *interpreted_format = interpret_ctrl_codes( mystrdup(format) );
 			if( use_val && arg_count == 1) 
 				printf( interpreted_format, val );
 			else if( var != NULL && arg_count == 1 ) 
@@ -2027,6 +2117,7 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 		{
 			xml_elem_t* sparm = NULL;
 			ASImage* imtmp = build_image_from_xml(asv, imman, fontman, ptr, &sparm, flags, verbose, display_win);
+			LOCAL_DEBUG_OUT("imtmp = %p", imtmp );
 			if (imtmp) 
 			{
 				if (tparm) xml_elem_delete(NULL, tparm);
@@ -2037,6 +2128,8 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 		else xml_elem_delete(NULL, tparm);
 	}
 
+	result = commit_xml_image_built( &state, id, result );
+#if 0	
 	if (id && result) 
 	{
     	char* buf = NEW_ARRAY(char, strlen(id) + 1 + 6 + 1);
@@ -2068,7 +2161,7 @@ build_image_from_xml( ASVisual *asv, ASImageManager *imman, ASFontManager *fontm
 			}
 		}
 	}
-
+#endif
 	return result;
 }
 
@@ -2457,7 +2550,7 @@ void reset_xml_buffer( ASXmlBuffer *xb )
 }	 
 
 
-inline void 
+void 
 add_xml_buffer_chars( ASXmlBuffer *xb, char *tmp, int len )
 {
 	if( xb->used + len > xb->allocated ) 
@@ -2689,19 +2782,19 @@ translate_special_sequence( const char *ptr, int len,  int *seq_len )
 	int c_len = 0 ;
 	if( ptr[0] == '&') 
 	{ 
-		if( 4 < len ) 
+		if( 4 <= len ) 
 		{	
 			c_len = 4 ;
 			if( strncmp(ptr,"&lt;", c_len ) == 0 ) c = '<' ;
 			else if( strncmp(ptr,"&gt;", c_len ) == 0 ) c = '>' ;
 		}
-		if( c != '\0' && 5 < len ) 
+		if( c == '\0' && 5 <= len ) 
 		{	
 			c_len = 5 ;
 			if( strncmp(ptr,"&amp;", c_len ) == 0 ) c = '&' ;
 		}
 				
-		if( c != '\0' && 6 < len ) 
+		if( c == '\0' && 6 <= len ) 
 		{	
 			c_len = 6 ;
 			if(      strncmp(ptr,"&quot;", c_len ) == 0 ) c = '"' ;
@@ -2713,7 +2806,7 @@ translate_special_sequence( const char *ptr, int len,  int *seq_len )
 			else if( strncmp(ptr,"&euro;", c_len ) == 0 ) c = '' ;					 
 		}
 
-		if( c != '\0' && 7 < len ) 
+		if( c == '\0' && 7 <= len ) 
 		{	
 			c_len = 7 ;
 			if( strncmp(ptr,"&OElig;", c_len ) == 0 ) c = '' ;
@@ -2728,7 +2821,7 @@ translate_special_sequence( const char *ptr, int len,  int *seq_len )
 			else if( strncmp(ptr,"&rdquo;", c_len ) == 0 ) c = '' ;
 			else if( strncmp(ptr,"&bdquo;", c_len ) == 0 ) c = '' ;
 		}				
-		if( c != '\0' && 8 < len ) 
+		if( c == '\0' && 8 <= len ) 
 		{	
 			c_len = 8 ;
 			if( strncmp(ptr,"&Scaron;", c_len ) == 0 ) c = '' ;
