@@ -108,7 +108,7 @@ typedef struct ASWharfButton
     int             folder_x, folder_y;
     unsigned int    folder_width, folder_height;
 
-    FunctionData   *fdata;
+    FunctionData   *fdata[Button5];
 
     struct ASWharfFolder   *folder;
     struct ASWharfFolder   *parent;
@@ -171,10 +171,16 @@ typedef struct ASWharfState
 
 	ASWharfButton *focused_button ;
 
+	FunctionData   *default_action[Button5];
+
 }ASWharfState;
 
 ASWharfState WharfState;
 WharfConfig *Config = NULL;
+int Rows_override = -1 ;
+int Columns_override = -1 ;
+
+Atom _AS_WHARF_CLOSE = None ;
 
 #define WHARF_BUTTON_EVENT_MASK   (ButtonReleaseMask |\
                                    ButtonPressMask | LeaveWindowMask | EnterWindowMask |\
@@ -194,10 +200,11 @@ ASWharfFolder *build_wharf_folder( WharfButton *list, ASWharfButton *parent, Boo
 Bool display_wharf_folder( ASWharfFolder *aswf, int left, int top, int right, int bottom );
 Bool display_main_folder();
 void withdraw_wharf_folder( ASWharfFolder *aswf );
+static inline void withdraw_wharf_subfolders( ASWharfFolder *aswf );
 void on_wharf_moveresize( ASEvent *event );
 void destroy_wharf_folder( ASWharfFolder **paswf );
 void on_wharf_pressed( ASEvent *event );
-void release_pressure();
+void release_pressure( int button );
 Bool check_pending_swallow( ASWharfFolder *aswf );
 void exec_pending_swallow( ASWharfFolder *aswf );
 void check_swallow_window( ASWindowData *wd );
@@ -219,18 +226,32 @@ void DeadPipe(int);
 int
 main (int argc, char **argv)
 {
-    /* Save our program name - for error messages */
+    int i;
+	/* Save our program name - for error messages */
 	set_DeadPipe_handler(DeadPipe);
     InitMyApp (CLASS_WHARF, argc, argv, NULL, NULL, 0 );
+	LinkAfterStepConfig();
+    for( i = 1 ; i< argc ; ++i)
+	{
+		LOCAL_DEBUG_OUT( "argv[%d] = \"%s\", original argv[%d] = \"%s\"", i, argv[i], i, MyArgs.saved_argv[i]);	  
+		if( argv[i] != NULL )
+		{ 	
+	    	if( strcmp( argv[i] , "--rows" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
+				Rows_override = atoi( argv[i+1] );
+	    	else if( strcmp( argv[i] , "--cols" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
+				Columns_override = atoi( argv[i+1] );
+		}
+	}
 
     memset( &WharfState, 0x00, sizeof(WharfState));
 
-    ConnectX( ASDefaultScr, PropertyChangeMask|EnterWindowMask );
+    ConnectX( ASDefaultScr, EnterWindowMask );
+	_AS_WHARF_CLOSE = XInternAtom( dpy, "_AS_WHARF_CLOSE", False );
     ConnectAfterStep (M_TOGGLE_PAGING |
                     M_NEW_DESKVIEWPORT |
                     M_END_WINDOWLIST |
                     WINDOW_CONFIG_MASK |
-                    WINDOW_NAME_MASK);
+                    WINDOW_NAME_MASK, 0);
     balloon_init (False);
 
     Config = CreateWharfConfig ();
@@ -341,10 +362,18 @@ CheckConfigSanity()
     if( Config == NULL )
         Config = CreateWharfConfig ();
 
+	if( MyArgs.geometry.flags != 0 ) 
+		Config->geometry = MyArgs.geometry ;
+
+	if( Rows_override >= 0 ) 
+		Config->rows = Rows_override ;
+	if( Columns_override >= 0 ) 
+		Config->columns = Columns_override ;
+
     if( Config->rows <= 0 && Config->columns <= 0 )
         Config->rows = 1;
-
-    mystyle_get_property (Scr.wmprops);
+    
+	mystyle_get_property (Scr.wmprops);
 
     sprintf( buf, "*%sTile", get_application_name() );
     LOCAL_DEBUG_OUT("Attempting to use style \"%s\"", buf);
@@ -612,6 +641,7 @@ process_message (send_data_type type, send_data_type *body)
 void
 DispatchEvent (ASEvent * event)
 {
+	static Bool root_pointer_moved = True ;
     SHOW_EVENT_TRACE(event);
 
     event->client = NULL ;
@@ -639,7 +669,7 @@ DispatchEvent (ASEvent * event)
             on_wharf_pressed( event );
             break;
         case ButtonRelease:
-            release_pressure();
+            release_pressure(event->x.xbutton.button);
 			{
 			    ASMagic *obj = fetch_object( event->w ) ;
 				if( obj && obj->magic == MAGIC_WHARF_BUTTON )
@@ -655,6 +685,7 @@ DispatchEvent (ASEvent * event)
 			}
             break;
         case MotionNotify :
+			root_pointer_moved = True ;
             break ;
         case EnterNotify :
 			if( event->x.xcrossing.window == Scr.Root )
@@ -672,7 +703,8 @@ DispatchEvent (ASEvent * event)
                 if( obj != NULL && obj->magic == MAGIC_WHARF_BUTTON )
                 {
                     ASWharfButton *aswb = (ASWharfButton*)obj;
-                    on_astbar_pointer_action( aswb->bar, 0, (event->x.type==LeaveNotify));
+                    on_astbar_pointer_action( aswb->bar, 0, (event->x.type==LeaveNotify), root_pointer_moved);
+					root_pointer_moved = False ;
 					if(event->x.type == EnterNotify)
 						change_button_focus(aswb, True ); 
                 }
@@ -696,11 +728,16 @@ DispatchEvent (ASEvent * event)
                 LOCAL_DEBUG_OUT("ClientMessage(\"%s\",data=(%lX,%lX,%lX,%lX,%lX)", name, event->x.xclient.data.l[0], event->x.xclient.data.l[1], event->x.xclient.data.l[2], event->x.xclient.data.l[3], event->x.xclient.data.l[4]);
                 XFree( name );
 #endif
-                if ( event->x.xclient.format == 32 &&
-                    event->x.xclient.data.l[0] == _XA_WM_DELETE_WINDOW )
-                {
-                    DeadPipe(0);
-                }
+                if ( event->x.xclient.format == 32 )
+				{	
+					if( event->x.xclient.data.l[0] == _XA_WM_DELETE_WINDOW )
+                	{
+                    	DeadPipe(0);
+                	}else if ( event->x.xclient.data.l[0] == _AS_WHARF_CLOSE )
+                	{
+						withdraw_wharf_subfolders( WharfState.root_folder );
+					}
+				}		  
             }
             break;
         case ReparentNotify :
@@ -712,7 +749,8 @@ DispatchEvent (ASEvent * event)
             break ;
 	    case PropertyNotify:
 			LOCAL_DEBUG_OUT( "property %s(%lX), _XROOTPMAP_ID = %lX, event->w = %lX, root = %lX", XGetAtomName(dpy, event->x.xproperty.atom), event->x.xproperty.atom, _XROOTPMAP_ID, event->w, Scr.Root );
-            if( event->x.xproperty.atom == _XROOTPMAP_ID && event->w == Scr.Root )
+			handle_wmprop_event (Scr.wmprops, &(event->x));
+            if( event->x.xproperty.atom == _AS_BACKGROUND )
             {
                 LOCAL_DEBUG_OUT( "root background updated!%s","");
 				clear_root_image_cache( WharfState.root_folder );
@@ -725,7 +763,6 @@ DispatchEvent (ASEvent * event)
             }else if( event->x.xproperty.atom == _AS_STYLE )
 			{
 				LOCAL_DEBUG_OUT( "AS Styles updated!%s","");
-				handle_wmprop_event (Scr.wmprops, &(event->x));
 				mystyle_list_destroy_all(&(Scr.Look.styles_list));
 				LoadColorScheme();
 				CheckConfigSanity();
@@ -877,17 +914,14 @@ build_wharf_button_tbar(WharfButton *wb)
                         get_flags( Config->set_flags, WHARF_BEVEL )? "set":"unset",
                         Config->bevel,
                         get_flags( Config->flags, WHARF_NO_BORDER )?"set":"unset" );
-    if( get_flags( Config->set_flags, WHARF_BEVEL ) )
+    if( get_flags( Config->flags, WHARF_NO_BORDER ) )
+	{	
+        set_astbar_hilite( bar, BAR_STATE_UNFOCUSED, 0 );
+        set_astbar_hilite( bar, BAR_STATE_FOCUSED, 0 );
+    }else if( get_flags( Config->set_flags, WHARF_BEVEL ) )
     {
-        if( get_flags( Config->flags, WHARF_NO_BORDER ) )
-		{	
-            set_astbar_hilite( bar, BAR_STATE_UNFOCUSED, 0 );
-            set_astbar_hilite( bar, BAR_STATE_FOCUSED, 0 );
-        }else
-		{	
-            set_astbar_hilite( bar, BAR_STATE_UNFOCUSED, Config->bevel );
-            set_astbar_hilite( bar, BAR_STATE_FOCUSED, Config->bevel );
-		}
+        set_astbar_hilite( bar, BAR_STATE_UNFOCUSED, Config->bevel );
+        set_astbar_hilite( bar, BAR_STATE_FOCUSED, Config->bevel );
     }else
 	{	
        	set_astbar_hilite( bar, BAR_STATE_UNFOCUSED, RIGHT_HILITE|BOTTOM_HILITE );
@@ -941,11 +975,14 @@ LOCAL_DEBUG_OUT( "contents %d has function %p with func = %ld", i, function, fun
 	        	if( function )
 				{
 					int func = function->func ;
-					if( IsSwallowFunc(func) || IsExecFunc(func) )
+					if( func != F_ExecInTerm && (IsSwallowFunc(func) || IsExecFunc(func)) )
 					{
 			   			disabled = (!is_executable_in_path (function->text));
 						if( disabled )
+						{
+							wb->contents[i].unavailable = True ;
 							show_warning( "Application \"%s\" cannot be found in the PATH.", function->text );
+						}
 					}else
 						disabled = False ;
 				}
@@ -985,8 +1022,6 @@ LOCAL_DEBUG_OUT( "contents %d has function %p with func = %ld", i, function, fun
         wb = list;
         while( wb )
         {
-			FunctionData *function = NULL ;
-
 			if( get_flags( wb->set_flags, WHARF_BUTTON_DISABLED ) )
 			{
 				wb = wb->next ;
@@ -995,21 +1030,43 @@ LOCAL_DEBUG_OUT( "contents %d has function %p with func = %ld", i, function, fun
 
             aswb->name = mystrdup( wb->title );
 			if( wb->contents )
-				function = wb->contents[wb->selected_content].function ;
-
-            if( function )
-            {
-                aswb->fdata = safecalloc( 1, sizeof(FunctionData) );
-                dup_func_data( aswb->fdata, function );
-                if( IsSwallowFunc(aswb->fdata->func) )
-                {
-                    set_flags( aswb->flags, ASW_SwallowTarget );
-                    register_swallow_target( aswb->fdata->name, aswb );
-                    if( aswb->fdata->func == F_MaxSwallow ||
-                        aswb->fdata->func == F_MaxSwallowModule )
-                        set_flags( aswb->flags, ASW_MaxSwallow );
-                }
-            }
+			{
+				int i ;
+				for( i = 0 ; i < wb->contents_num ; ++i )
+					if( !wb->contents[i].unavailable )
+					{
+						FunctionData *function = wb->contents[i].function ;
+			            if( function )
+            			{
+							int btn = Button1 ;
+							if( function->name != NULL && strlen(function->name) == 1 )
+							{
+								switch( function->name[0] )
+								{
+									case 'l' : btn = Button1 ;    break ;	
+									case 'm' : btn = Button2 ;    break ;	
+									case 'r' : btn = Button3 ;    break ;	
+									case '4' : btn = Button4 ;	  break ;
+									case '5' : btn = Button5 ;	  break ;
+								}		   
+							}	  
+							btn -= Button1 ;
+							if( aswb->fdata[btn] == NULL ) 
+							{	
+                				aswb->fdata[btn] = safecalloc( 1, sizeof(FunctionData) );
+                				dup_func_data( aswb->fdata[btn], function );
+                				if( IsSwallowFunc(aswb->fdata[btn]->func) )
+                				{
+                    				set_flags( aswb->flags, ASW_SwallowTarget );
+                    				register_swallow_target( aswb->fdata[btn]->name, aswb );
+                    				if( aswb->fdata[btn]->func == F_MaxSwallow ||
+                        				aswb->fdata[btn]->func == F_MaxSwallowModule )
+                        				set_flags( aswb->flags, ASW_MaxSwallow );
+                				}
+							}
+            			}
+					}
+			}
 			/* TODO:	Transient buttons are just spacers - they should not 
 			 * 			have any balloons displayed on them , nor they should 
 			 * 			interpret any clicks
@@ -1085,13 +1142,21 @@ destroy_wharf_folder( ASWharfFolder **paswf )
         while( --i >= 0 )
         {
             ASWharfButton *aswb = &(aswf->buttons[i]);
+			int i ;
+			if( aswb->swallowed ) 
+			{	
+				send_wm_protocol_request(aswb->swallowed->current->w, _XA_WM_DELETE_WINDOW, CurrentTime);
+
+			}
             if( aswb->name )
                 free( aswb->name );
             destroy_astbar(&(aswb->bar));
             if( aswb->canvas )
                 unregister_object( aswb->canvas->w );
             destroy_ascanvas(&(aswb->canvas));
-            destroy_func_data(&(aswb->fdata));
+			for( i = 0 ; i < 5 ; ++i ) 
+				if( aswb->fdata[i] )
+            		destroy_func_data(&(aswb->fdata[i]));
             destroy_wharf_folder(&(aswb->folder));
         }
         free( aswf->buttons );
@@ -1220,6 +1285,7 @@ update_wharf_folder_transprency( ASWharfFolder *aswf, Bool force )
 			ASWharfButton *aswb= &(aswf->buttons[i]);
 			if( update_astbar_transparency( aswb->bar, aswb->canvas, force ) )
 			{
+				invalidate_canvas_save( aswb->canvas );
 				render_wharf_button( aswb );
 				update_canvas_display( aswb->canvas );
 			}
@@ -1236,12 +1302,16 @@ change_button_focus(ASWharfButton *aswb, Bool focused )
 	
 	if( aswb == NULL && focused && Scr.Look.MSWindow[BACK_FOCUSED] == NULL ) 
 		return ;
-	
-	set_astbar_focused( aswb->bar, NULL, focused );			   
-	render_wharf_button( aswb );
-	update_canvas_display( aswb->canvas );
-	update_wharf_folder_shape( aswb->parent );
 
+	if( set_astbar_focused( aswb->bar, NULL, focused ) )
+	{	
+		if( !swap_save_canvas( aswb->canvas ) )	   
+		{		
+			render_wharf_button( aswb );
+		}
+		update_canvas_display( aswb->canvas );
+		update_wharf_folder_shape( aswb->parent );
+	}
 	if( focused ) 
 		WharfState.focused_button = aswb ;
 	else if( WharfState.focused_button == aswb )
@@ -1260,6 +1330,7 @@ update_wharf_folder_styles( ASWharfFolder *aswf, Bool force )
 			set_astbar_style_ptr( aswb->bar, BAR_STATE_FOCUSED, Scr.Look.MSWindow[Scr.Look.MSWindow[BACK_FOCUSED]?BACK_FOCUSED:BACK_UNFOCUSED] );
 			if( set_astbar_style_ptr( aswb->bar, BAR_STATE_UNFOCUSED, Scr.Look.MSWindow[BACK_UNFOCUSED] ))
 			{
+				invalidate_canvas_save( aswb->canvas );
 				render_wharf_button( aswb );
 				update_canvas_display( aswb->canvas );
 			}
@@ -1281,7 +1352,7 @@ map_wharf_folder( ASWharfFolder *aswf,
 	set_folder_name( aswf, False );
 
     shints.flags = USSize|PWinGravity;
-    if( get_flags( Config->set_flags, WHARF_GEOMETRY ) )
+    if( get_flags( Config->set_flags, WHARF_GEOMETRY ) || get_flags(MyArgs.geometry.flags, XValue|YValue))
         shints.flags |= USPosition ;
     else
         shints.flags |= PPosition ;
@@ -1634,6 +1705,11 @@ display_wharf_folder( ASWharfFolder *aswf, int left, int top, int right, int bot
         (get_flags( aswf->flags, ASW_Mapped ) && !get_flags( aswf->flags, ASW_Withdrawn )) )
         return False;
 
+	if( MyArgs.gravity == SouthWestGravity || MyArgs.gravity == SouthEastGravity )
+		south = True ;
+	if( MyArgs.gravity == NorthEastGravity || MyArgs.gravity == SouthEastGravity )
+		east = True ;
+
 	if( aswf != WharfState.root_folder )
 	{
   		if( get_flags( aswf->flags, ASW_Vertical ) )
@@ -1809,9 +1885,6 @@ LOCAL_DEBUG_OUT( "unmapping canvas %p at %dx%d%+d%+d", aswf->canvas, aswf->canva
 	unmap_wharf_subfolders( aswf );
 }
 
-static inline void
-withdraw_wharf_subfolders( ASWharfFolder *aswf );
-
 void
 withdraw_wharf_folder( ASWharfFolder *aswf )
 {
@@ -1849,7 +1922,22 @@ withdraw_wharf_subfolders( ASWharfFolder *aswf )
     int i = aswf->buttons_num;
     while ( --i >= 0 )
     {
-        if( aswf->buttons[i].folder &&
+        if( aswf->buttons[i].swallowed ) 
+		{
+			ASCanvas *sc = aswf->buttons[i].swallowed->current ;
+			if( sc )
+			{	
+				XClientMessageEvent ev;
+				ev.type = ClientMessage;
+				ev.window = sc->w;
+				ev.message_type = _AS_WHARF_CLOSE;
+				ev.format = 32;
+				ev.data.l[0] = _AS_WHARF_CLOSE;
+				ev.data.l[1] = CurrentTime;
+    			XSendEvent (dpy, sc->w, False, 0, (XEvent *) & ev);
+			}
+		}	 
+		if( aswf->buttons[i].folder &&
             get_flags( aswf->buttons[i].folder->flags, ASW_Mapped ) )
             withdraw_wharf_folder( aswf->buttons[i].folder );
     }
@@ -1899,10 +1987,15 @@ exec_pending_swallow( ASWharfFolder *aswf )
         while( --i >= 0 )
         {
             if( get_flags(aswf->buttons[i].flags, ASW_SwallowTarget ) &&
-                aswf->buttons[i].fdata &&
                 aswf->buttons[i].swallowed == NULL )
             {
-                SendCommand( aswf->buttons[i].fdata, 0);
+				int k ; 
+				for( k = 0 ; k < 5 ; ++k ) 
+                	if( aswf->buttons[i].fdata[k] && IsSwallowFunc(aswf->buttons[i].fdata[k]->func)) 
+					{
+                		SendCommand( aswf->buttons[i].fdata[k], 0);
+						break;
+					}
             }
             if( aswf->buttons[i].folder )
                 exec_pending_swallow( aswf->buttons[i].folder );
@@ -1911,69 +2004,60 @@ exec_pending_swallow( ASWharfFolder *aswf )
 }
 
 void
-grab_swallowed_canvas_btns( ASCanvas *canvas, Bool action, Bool withdraw )
+grab_swallowed_canvas_btn( ASCanvas *canvas, int button, int non_lock_mods )
 {
 	register int i = 0 ;
 	register unsigned int *mods = lock_mods;
 
-	LOCAL_DEBUG_OUT( "%p,%d,%d", canvas, action, withdraw );
+	LOCAL_DEBUG_OUT( "%p,%d,0x%x", canvas, button, non_lock_mods );
 
 	if( AS_ASSERT(canvas) )
 		return;
     do
     {
         /* grab button 1 if this button performs an action */
-        if( action )
-		{	
-            XGrabButton (dpy, Button1, mods[i],
-                        canvas->w,
-                        False, ButtonPressMask | ButtonReleaseMask,
-                        GrabModeAsync, GrabModeAsync, None, None);
+        XGrabButton(dpy, button, mods[i]|non_lock_mods,
+	                canvas->w,
+                    False, ButtonPressMask | ButtonReleaseMask,
+                    GrabModeAsync, GrabModeAsync, None, None);
 #if !defined(NO_DEBUG_OUTPUT)			
 			ASSync(False);
 			fprintf( stderr, "line = %d, mods = 0x%X\n", __LINE__, mods[i] );
 #endif
-		}
-        /* grab button 3 if this is the root folder */
-        if (withdraw )
-		{	
-            XGrabButton (dpy, Button3, mods[i],
-                        canvas->w,
-                        False, ButtonPressMask | ButtonReleaseMask,
-                        GrabModeAsync, GrabModeAsync, None, None);
-#if !defined(NO_DEBUG_OUTPUT)			   
-			ASSync(False);
-			fprintf( stderr, "line = %d, canvas = %p, window = %lX, i = %d, mods = 0x%X\n", __LINE__, canvas, canvas->w, i, mods[i] );
-#endif
-		}
 		if( mods[i] == 0 )
 			return;
     }while (++i < MAX_LOCK_MODS );
 
-	if( action )
-	{	
-        XGrabButton (dpy, Button1, 0,
-                    canvas->w,
-                    False, ButtonPressMask | ButtonReleaseMask,
-                    GrabModeAsync, GrabModeAsync, None, None);
+    XGrabButton(dpy, button, non_lock_mods,
+                canvas->w,
+                False, ButtonPressMask | ButtonReleaseMask,
+                GrabModeAsync, GrabModeAsync, None, None);
 #if !defined(NO_DEBUG_OUTPUT)			   		
 		ASSync(False);
 		fprintf( stderr, "line = %d, mods = 0\n", __LINE__ );
 #endif
-	}
-    /* grab button 3 if this is the root folder */
-    if (withdraw )
-	{	
-        XGrabButton (dpy, Button3, 0,
-                    canvas->w,
-                    False, ButtonPressMask | ButtonReleaseMask,
-                    GrabModeAsync, GrabModeAsync, None, None);
-#if !defined(NO_DEBUG_OUTPUT)			   			
-			ASSync(False);
-			fprintf( stderr, "line = %d, mods = 0", __LINE__ );
-#endif
-	}
+}
 
+
+void
+grab_swallowed_canvas_btns( ASCanvas *canvas, ASWharfButton *aswb, Bool withdraw_btn )
+{	
+	if( aswb->folder!=NULL )
+		grab_swallowed_canvas_btn( canvas, Button1, 0);
+
+	if( withdraw_btn && aswb->parent == WharfState.root_folder )
+	{	
+		grab_swallowed_canvas_btn( canvas, Button3, ControlMask);
+		grab_swallowed_canvas_btn( canvas, Button3, 0);
+	}else if( aswb->fdata[Button3-Button1] ) 
+		grab_swallowed_canvas_btn( canvas, Button3, 0);
+	
+	if( aswb->fdata[Button2-Button1] ) 
+		grab_swallowed_canvas_btn( canvas, Button2, 0);
+	if( aswb->fdata[Button4-Button1] ) 
+		grab_swallowed_canvas_btn( canvas, Button4, 0);
+	if( aswb->fdata[Button5-Button1] ) 
+		grab_swallowed_canvas_btn( canvas, Button5, 0);
 }
 
 void
@@ -2054,8 +2138,8 @@ check_swallow_window( ASWindowData *wd )
 	}
     if( w != None )
         w = get_parent_window( w );
-    LOCAL_DEBUG_OUT( "second parent %lX, root %lX", w, Scr.Root );
-    if( w != Scr.Root )
+    LOCAL_DEBUG_OUT( "second parent %lX, root %lX, service_win = %lX", w, Scr.Root, Scr.ServiceWin );
+    if( w != Scr.Root && w != Scr.ServiceWin)
 	{
 		ungrab_server();
 		return ;
@@ -2075,7 +2159,7 @@ check_swallow_window( ASWindowData *wd )
     ungrab_server();
 	ASSync(False);
     sleep_a_millisec(100);
-	grab_swallowed_canvas_btns( nc, (aswb->folder!=NULL), withdraw_btn && aswb->parent == WharfState.root_folder);
+	grab_swallowed_canvas_btns( nc, aswb, withdraw_btn );
 
     if( get_flags( wd->flags, AS_ClientIcon ) && !get_flags( wd->flags, AS_ClientIconPixmap) &&
 		wd->icon != None )
@@ -2086,7 +2170,7 @@ check_swallow_window( ASWindowData *wd )
         register_object( wd->icon, (ASMagic*)aswb );
         XSelectInput (dpy, wd->icon, StructureNotifyMask);
 		ASSync(False);
-        grab_swallowed_canvas_btns(  ic, (aswb->folder!=NULL), withdraw_btn && aswb->parent == WharfState.root_folder);
+		grab_swallowed_canvas_btns( ic, aswb, withdraw_btn );
     }
     aswb->swallowed->current = ( get_flags( wd->state_flags, AS_Iconic ) &&
                                     aswb->swallowed->iconic != NULL )?
@@ -2212,8 +2296,10 @@ on_wharf_button_moveresize( ASWharfButton *aswb, ASEvent *event )
         update_astbar_transparency( aswb->bar, aswb->canvas, False );
 
     if( changes != 0 && (DoesBarNeedsRendering(aswb->bar) || is_canvas_needs_redraw( aswb->canvas )))
+	{
+		invalidate_canvas_save( aswb->canvas );
         render_wharf_button( aswb );
-
+	}
 #ifndef SHAPE
     swallowed_changes = 0 ;                    /* if no shaped extentions - ignore the changes */
 #endif
@@ -2541,13 +2627,14 @@ press_wharf_button( ASWharfButton *aswb, int state )
 }
 
 void
-release_pressure()
+release_pressure( int button )
 {
     ASWharfButton *pressed = WharfState.pressed_button ;
 LOCAL_DEBUG_OUT( "pressed button is %p", pressed );
+	button -= Button1 ;
     if( pressed )
     {
-        if( pressed->folder )
+        if( pressed->folder && !(button > 0 && pressed->fdata[button]))
         {
 LOCAL_DEBUG_OUT( "pressed button has folder %p (%s)", pressed->folder, get_flags( pressed->folder->flags, ASW_Mapped )?"Mapped":"Unmapped" );
             if( get_flags( pressed->folder->flags, ASW_Mapped ) )
@@ -2556,16 +2643,18 @@ LOCAL_DEBUG_OUT( "pressed button has folder %p (%s)", pressed->folder, get_flags
                 display_wharf_folder( pressed->folder, pressed->canvas->root_x, pressed->canvas->root_y,
                                                        pressed->canvas->root_x+pressed->canvas->width,
                                                        pressed->canvas->root_y+pressed->canvas->height  );
-        }else if( pressed->fdata )
+        }else if( pressed->fdata[button] )
         {
 #if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
-            print_func_data(__FILE__, __FUNCTION__, __LINE__, pressed->fdata);
+            print_func_data(__FILE__, __FUNCTION__, __LINE__, pressed->fdata[button]);
 #endif
-            if( !get_flags( pressed->flags, ASW_SwallowTarget ) || pressed->swallowed == NULL )
+            if( button > 0 ||
+				!get_flags( pressed->flags, ASW_SwallowTarget ) || 
+				pressed->swallowed == NULL )
             {  /* send command to the AS-proper : */
 				ASWharfFolder *parentf = pressed->parent ;
 				ASWharfButton *parentb = NULL ;
-                SendCommand( pressed->fdata, 0);
+                SendCommand( pressed->fdata[button], 0);
 				while( parentf != WharfState.root_folder )
 				{
 					parentb = parentf->parent ;
@@ -2615,73 +2704,80 @@ on_wharf_pressed( ASEvent *event )
             if( (WITHDRAW_ON_EDGE(Config) && (&(aswf->buttons[0]) == aswb || &(aswf->buttons[aswf->buttons_num-1]) == aswb)) ||
                  WITHDRAW_ON_ANY(Config))
         	{
-            	if( get_flags( WharfState.root_folder->flags, ASW_Withdrawn) )
-				{
-					/* update our name to normal */	  
-					set_folder_name( WharfState.root_folder, False );
-					LOCAL_DEBUG_OUT( "un - withdrawing folder%s","" );
-                	display_main_folder();
-            	}else
-            	{
-                	int wx = 0, wy = 0, wwidth, wheight;
-                	int i = aswf->buttons_num ;
+				if( aswb->fdata[Button3-Button1] == NULL || 
+					(event->x.xbutton.state&AllModifierMask) == ControlMask )
+				{			   
+            		if( get_flags( WharfState.root_folder->flags, ASW_Withdrawn) )
+					{
+						/* update our name to normal */	  
+						set_folder_name( WharfState.root_folder, False );
+						LOCAL_DEBUG_OUT( "un - withdrawing folder%s","" );
+                		display_main_folder();
+            		}else
+            		{
+                		int wx = 0, wy = 0, wwidth, wheight;
+                		int i = aswf->buttons_num ;
                 
-					if( Config->withdraw_style < WITHDRAW_ON_ANY_BUTTON_AND_SHOW )
-                    	aswb = &(aswf->buttons[0]);
+						if( Config->withdraw_style < WITHDRAW_ON_ANY_BUTTON_AND_SHOW )
+                    		aswb = &(aswf->buttons[0]);
 
-					withdraw_wharf_subfolders( aswf );
-					/* update our name to withdrawn */	  					
-					set_folder_name( aswf, True );
+						withdraw_wharf_subfolders( aswf );
+						/* update our name to withdrawn */	  					
+						set_folder_name( aswf, True );
 
-                	wwidth = aswb->desired_width ;
-                	wheight = aswb->desired_height ;
-                	if( get_flags( Config->geometry.flags, XNegative ) )
-                    	wx = Scr.MyDisplayWidth - wwidth ;
-                	if( get_flags( Config->geometry.flags, YNegative ) )
-                    	wy = Scr.MyDisplayHeight - wheight ;
+                		wwidth = aswb->desired_width ;
+                		wheight = aswb->desired_height ;
+                		if( get_flags( Config->geometry.flags, XNegative ) )
+                    		wx = Scr.MyDisplayWidth - wwidth ;
+                		if( get_flags( Config->geometry.flags, YNegative ) )
+                    		wy = Scr.MyDisplayHeight - wheight ;
 
-					if( get_flags(Config->flags, WHARF_ANIMATE ) )
-					{	
-						set_flags(aswf->flags,ASW_UseBoundary );
-						animate_wharf_loop(aswf, aswf->canvas->width, aswf->canvas->height, wwidth, wheight );
-						clear_flags(aswf->flags,ASW_UseBoundary );
-					}
+						if( get_flags(Config->flags, WHARF_ANIMATE ) )
+						{	
+							set_flags(aswf->flags,ASW_UseBoundary );
+							animate_wharf_loop(aswf, aswf->canvas->width, aswf->canvas->height, wwidth, wheight );
+							clear_flags(aswf->flags,ASW_UseBoundary );
+						}
 
-					LOCAL_DEBUG_OUT( "withdrawing folder to %dx%d%+d%+d", wwidth, wheight, wx, wy );
-                	XRaiseWindow( dpy, aswb->canvas->w );
-                	while ( --i >= 0 )
-                	{
-                    	if( &(aswf->buttons[i]) != aswb && 
-							aswf->buttons[i].canvas->root_x == aswf->canvas->root_x && 
-							aswf->buttons[i].canvas->root_y == aswf->canvas->root_y)
-                    	{
-                        	aswf->buttons[i].folder_x = wwidth ;
-                        	aswf->buttons[i].folder_y = wheight ;
-                        	move_canvas( aswf->buttons[i].canvas, wwidth, wheight );
-                    	}
-                	}
-                	set_flags( aswf->flags, ASW_Withdrawn );
-					set_withdrawn_clip_area( aswf, wx, wy, wwidth, wheight );
-                	moveresize_canvas( aswf->canvas, wx, wy, wwidth, wheight );
-//					ASSync(False);
-//					MapConfigureNotifyLoop();
+						LOCAL_DEBUG_OUT( "withdrawing folder to %dx%d%+d%+d", wwidth, wheight, wx, wy );
+                		XRaiseWindow( dpy, aswb->canvas->w );
+                		while ( --i >= 0 )
+                		{
+                    		if( &(aswf->buttons[i]) != aswb && 
+								aswf->buttons[i].canvas->root_x == aswf->canvas->root_x && 
+								aswf->buttons[i].canvas->root_y == aswf->canvas->root_y)
+                    		{
+                        		aswf->buttons[i].folder_x = wwidth ;
+                        		aswf->buttons[i].folder_y = wheight ;
+                        		move_canvas( aswf->buttons[i].canvas, wwidth, wheight );
+                    		}
+                		}
+                		set_flags( aswf->flags, ASW_Withdrawn );
+						set_withdrawn_clip_area( aswf, wx, wy, wwidth, wheight );
+                		moveresize_canvas( aswf->canvas, wx, wy, wwidth, wheight );
+	//					ASSync(False);
+	//					MapConfigureNotifyLoop();
 
-                	aswb->folder_x = 0;
-                	aswb->folder_y = 0;
-                	aswb->folder_width = wwidth ;
-                	aswb->folder_height = wheight ;
-                	moveresize_canvas( aswb->canvas, 0, 0, wwidth, wheight );
+                		aswb->folder_x = 0;
+                		aswb->folder_y = 0;
+                		aswb->folder_width = wwidth ;
+                		aswb->folder_height = wheight ;
+                		moveresize_canvas( aswb->canvas, 0, 0, wwidth, wheight );
 
-					aswf->withdrawn_button = aswb ;
-            	}
-            	return;
+						aswf->withdrawn_button = aswb ;
+            		}
+	            	return;
+				}
         	}
-			if( check_app_click	( aswb, &(event->x.xbutton) ) ) 
-			{
-	            event->x.xbutton.window = aswb->swallowed->current->w;
-				XSendEvent (dpy, aswb->swallowed->current->w, False, ButtonPressMask, &(event->x));	
-				return;
-			}	 
+			if( event->x.xbutton.button == Button1 || aswb->fdata[event->x.xbutton.button-Button1] == NULL ) 
+			{	
+				if( check_app_click	( aswb, &(event->x.xbutton) ) ) 
+				{
+	            	event->x.xbutton.window = aswb->swallowed->current->w;
+					XSendEvent (dpy, aswb->swallowed->current->w, False, ButtonPressMask, &(event->x));	
+					return;
+				}	 
+			}
 		}
         press_wharf_button( aswb, event->x.xbutton.state );
     }
