@@ -142,9 +142,14 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
     ASWindow *asw = (ASWindow*)data ;
 	LOCAL_DEBUG_OUT( "window \"%s\", is a %smodule", ASWIN_NAME(asw), ASWIN_HFLAGS(asw,AS_Module)?
 	" ":"non ");
-    if( asw && swad && !ASWIN_HFLAGS(asw,AS_Module) )
+    if( asw && swad )
     {
 		Bool same_host = (asw->hints->client_host == NULL || mystrcasecmp( asw->hints->client_host, swad->this_host )== 0);
+
+		/* don't want to save modules - those are started from autoexec anyways */
+		if( ASWIN_HFLAGS(asw,AS_Module) ) 
+			return True;
+
 		if( asw->hints->client_cmd == NULL && same_host )
 		{
 		 	if( ASWIN_HFLAGS(asw, AS_PID) && asw->hints->pid > 0 )
@@ -213,21 +218,36 @@ make_aswindow_cmd_iter_func(void *data, void *aux_data)
 				free( cmd_app );	
 			}else
 	            fprintf( swad->f, 	"\tExec \"I:%s\" %s -geometry %s &\n", app_name, asw->hints->client_cmd, geom );
-			fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
-						        ", Layer %d"
-								", %s"
-								", StartsOnDesk %d"
-								", ViewportX %d"
-								", ViewportY %d"
-								", %s"
-								"\n",
-						     	app_name, pure_geometry,
-							 	ASWIN_LAYER(asw),
-							 	ASWIN_GET_FLAGS(asw,AS_Sticky)?"Sticky":"Slippery",
-								ASWIN_DESK(asw),
-								asw->status->viewport_x,
-								asw->status->viewport_y,
-								ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+			if( ASWIN_GET_FLAGS(asw,AS_Sticky) )
+			{	
+				fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
+							        ", Layer %d"
+									", Sticky"
+									", StartsOnDesk %d"
+									", %s"
+									"\n",
+						     		app_name, pure_geometry,
+							 		ASWIN_LAYER(asw),
+									ASWIN_DESK(asw),
+									ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+			}else
+			{
+				
+				fprintf( swad->f, 	"\tWait \"I:%s\" DefaultGeometry %s"
+							        ", Layer %d"
+									", Slippery"
+									", StartsOnDesk %d"
+									", ViewportX %d"
+									", ViewportY %d"
+									", %s"
+									"\n",
+						     		app_name, pure_geometry,
+							 		ASWIN_LAYER(asw),
+									ASWIN_DESK(asw),
+									((asw->status->x + asw->status->viewport_x) / Scr.MyDisplayWidth)*Scr.MyDisplayWidth,
+									((asw->status->y + asw->status->viewport_y) / Scr.MyDisplayHeight)*Scr.MyDisplayHeight,
+									ASWIN_GET_FLAGS(asw,AS_Iconic)?"StartIconic":"StartNormal");
+			}	 
 			if( pure_geometry ) 
 				free( pure_geometry );
 			if( geom )
@@ -903,7 +923,7 @@ restack_window_list( int desk, Bool send_msg_only )
         ASWindow **members = PVECTOR_HEAD(ASWindow*,l[i]->members);
         if( end_k > ids->allocated )
             end_k = ids->allocated ;
-        LOCAL_DEBUG_OUT( "layer %ld, end_k = %d", i, end_k );
+        LOCAL_DEBUG_OUT( "layer[%ld] = %d, end_k = %d", i, l[i]->layer, end_k );
         for( k = 0 ; k < end_k ; k++ )
         {
             register ASWindow *asw = members[k] ;
@@ -933,12 +953,15 @@ restack_window_list( int desk, Bool send_msg_only )
         {
             register int k, end_k = PVECTOR_USED(l[i]->members) ;
             register ASWindow **members = PVECTOR_HEAD(ASWindow*,l[i]->members);
-            if( end_k > ids->allocated )
-                end_k = ids->allocated ;
-            LOCAL_DEBUG_OUT( "layer %ld, end_k = %d", i, end_k );
+            if( windows_num+end_k > ids->allocated )
+                end_k = ids->allocated - windows_num ;
+            LOCAL_DEBUG_OUT( "layer[%ld] = %d, end_k = %d", i, l[i]->layer, end_k );
             for( k = 0 ; k < end_k ; k++ )
                 if( ASWIN_DESK(members[k]) == desk && !ASWIN_GET_FLAGS(members[k], AS_Dead))
+				{
                     windows[windows_num++] = get_window_frame(members[k]);
+					LOCAL_DEBUG_OUT( "\t windows[%d] = %lX", k, windows[windows_num-1] );
+				}
         }
 
         LOCAL_DEBUG_OUT( "Setting stacking order: windows_num = %ld, ", windows_num );
@@ -1214,6 +1237,7 @@ hide_focus()
     Scr.Windows->focused = NULL;
     Scr.Windows->ungrabbed = NULL;
     XRaiseWindow(dpy, Scr.ServiceWin);
+	LOCAL_DEBUG_OUT( "XSetInputFocus(window= %lX (service_win), time = %lu)", Scr.ServiceWin, Scr.last_Timestamp);
     XSetInputFocus (dpy, Scr.ServiceWin, RevertToParent, Scr.last_Timestamp);
     XSync(dpy, False );
 }
@@ -1264,10 +1288,20 @@ focus_window( ASWindow *asw, Window w )
     LOCAL_DEBUG_OUT( "focusing window %lX, client %lX, frame %lX, asw %p", w, asw->w, asw->frame, asw );
 	/* using last_Timestamp here causes problems when moving between screens */
 	/* at the same time using CurrentTime all the time seems to cause some apps to fail,
-	 * most noticeably GTK-perl */
+	 * most noticeably GTK-perl 
+	 * 
+	 * Take 2: disabled CurrentTime altogether as it screwes up focus handling
+	 * Basically if you use CurrentTime when there are still bunch of Events 
+	 * in the queue, those evens will not have any effect if you try setting 
+	 * focus using their time, as X aready used its own friggin current time.
+	 * Don't ask, its a mess.
+	 * */
 	if( w != None )
-	    XSetInputFocus ( dpy, w, RevertToParent, 
-						(Scr.Windows->focused == NULL)?CurrentTime:Scr.last_Timestamp);
+	{
+		Time t = /*(Scr.Windows->focused == NULL)?CurrentTime:*/Scr.last_Timestamp ;
+		LOCAL_DEBUG_OUT( "XSetInputFocus(window= %lX, time = %lu)", w, t);	  
+	    XSetInputFocus ( dpy, w, RevertToParent, t );
+	}
 
     ASSync(False );
     return (w!=None);
@@ -1548,6 +1582,10 @@ make_desk_winlist_menu(  ASWindowList *list, int desk, int sort_order, Bool icon
 			{
                 fdata.func = F_RAISE_IT ;
                 fdata.name = mystrdup(icon_name? ASWIN_ICON_NAME(clients[i]) : ASWIN_NAME(clients[i]));
+				if( !icon_name )
+	 				fdata.name_encoding = ASWIN_NAME_ENCODING(clients[i]) ;
+				LOCAL_DEBUG_OUT( "item's encoding = %d, name = \"%s\"", fdata.name_encoding,
+				fdata.name );
                 fdata.func_val[0] = (long) clients[i];
                 fdata.func_val[1] = (long) clients[i]->w;
 		if (++scut == ('9' + 1))
@@ -1567,6 +1605,8 @@ make_desk_winlist_menu(  ASWindowList *list, int desk, int sort_order, Bool icon
 		sfdata = safecalloc(1, sizeof(FunctionData));
                 sfdata->func = F_RAISE_IT;
                 sfdata->name = mystrdup(icon_name ? ASWIN_ICON_NAME(clients[i]) : ASWIN_NAME(clients[i]));
+				if( !icon_name )
+	 				sfdata->name_encoding = ASWIN_NAME_ENCODING(clients[i]) ;
                 sfdata->func_val[0] = (long) clients[i];
                 sfdata->func_val[1] = (long) clients[i]->w;
 		if (++scut == ('9' + 1))
